import sys
import argparse

# By AWARE on RC: python3.6 and it happy this *local* version of dataclasses, but ver>3.6 should go with system-wide version
try:
    from dataclasses import dataclass
except:
    from dataclasses_local import dataclass

from pathlib     import Path
# from unittest import mock # TODO
from pyremoter.services.remote_interfaces import RemoterInterface
from pyremoter.remoter         import Remoter
from pyremoter.services.status import Status


def parse_args():
    parser: argparse.ArgumentParser = argparse.ArgumentParser(
        description=\
            "PyRemoter handle for your uploading/downloading process for the range of connection types. See the *connection type* section.",
        )
    # arguments
    parser.add_argument("--conn_type", "-t", type=str,  required=True,  help="The connection type. Options: ssh")
    parser.add_argument("--command",   "-c", type=str,  required=True,  help="The command. Option: upload, download")
    
    parser.add_argument("--username",        type=str,  required=True,  help="Host username eg. nucky")
    parser.add_argument("--host",            type=str,  required=True,  help="Host address eg. 192.168.39.1")
    # No need to use for current state # parser.add_argument("--keys",      "-k", type=Path, required=True,  help="The path/to/key/username.pub. If not provided will be prompt to provide it.")
    # 
    parser.add_argument('--debug', action='store_true', help="debug")
    parser.add_argument('--check', action='store_true', help="check flag")

    parser.add_argument("--execute",   "-x", type=str,  required=False,  help="The remote command to execute")
    parser.add_argument("--src",             type=Path, required=False,  help="Source: file/folder")
    parser.add_argument("--dst",             type=Path, required=False,  help="Destination: file/folder")

    args: argparse.Namespace = parser.parse_args()

    # upload/download with --src and --dst
    if args.command.lower() in ["upload", "download"]:
        if args.src is None or args.dst is None:
            parser.error("--command with 'upload/download' requires --src and --dst.")
    
    # execute not --src and --dst
    if args.command.lower() in ["execute"]:
        if args.src is not None or args.dst is not None:
            parser.error("--command with 'execute' conflicts with --src and --dst.")

    return args

def is_ok(args: argparse.ArgumentParser) -> bool:
    ok: bool = True
#     # check wether it exists
#     if args.keys is not None:
#         if not args.keys.exists(): 
#             print(f"The key.pub does not exist. Aborting.")
#             ok = False

    return ok

    
@dataclass
class InfoHandler():
    # Input
    args: argparse.Namespace = None
    # Outputs
    _username : str          = None
    _host     : str          = None

    def __post_init__(self):
        # checks
        if self.args is None: raise Exception("You need to provide the path to `key.pub`")
        # username
        self._username = self._extract_username()
        self._psw      = self._extract_psw()
        # host
        self._host     = self.args.host

    '''
    Extracting the username:
    either /path/to/key/nucky.pub [one way for now]
    # TODO: or     /path/to/key/key.pub (parsing)
    '''
    def _extract_username(self) -> str:
        return self.args.username
        # return self.args.keys.name.split(".pub")[0] # minus .pub
    
    '''
    : TODO: extracting passwords from wherever
    '''
    def _extract_psw(self) -> str:
        return ""

    def get_username(self) -> str:
        return self._username
    
    def get_psw(self) -> str:
        return self._psw
    
    def get_host(self) -> str:
        return self._host


def main():
    args: argparse.Namespace = parse_args()
    if args.debug: print(args)
    if args.check: sys.exit(0)
    if not is_ok(args=args): sys.exit(1)

    ok: bool = True
    h = InfoHandler(args=args)
        
    r : Remoter = Remoter(
        verbose         = False,
        debug           = args.debug,
        connection_type = args.conn_type.lower(),
        host            = h.get_host(),
        username        = h.get_username(),
        psw             = h.get_psw(),
    )
    s: Status = r.connect()

    if   args.command.lower() == "upload":   s: Status = r.upload(src_fname = args.src, dst_fname=args.dst)
    elif args.command.lower() == "download": s: Status = r.download(src_fname = args.src, dst_fname=args.dst)
    elif args.command.lower() == "execute":
        _conn: RemoterInterface = r.share_connection()
        if _conn.__class__.__name__ == "Scp_wrap":
            if args.execute is not None:
                _conn.execute(args.execute)
        else:
            print("Nope")
    if not s.is_ok(): ok = False
    print(f"[INFO] Transfer status: [{s.get_status()}||{s.get_output_msg()}]")

    r.disconnect()
    
    # the end
    if not ok: sys.exit(1)


### TESTS

def test_InfoHandler_get_name():
    args = argparse.Namespace(keys=Path("/path/to/my/favorite/key/nucky.pub"), host="192.168.39.1")
    i = InfoHandler(args=args)
    assert i.get_username() == "nucky"